@@ -1,6 +1,8 @@
 import requests
+from jose.utils import base64url_decode
 
 from . import DEFAULT_API_URL, logger, ArweaveException
+from .utils import response_stream_to_file_object
 
 class HTTPClient:
     def __init__(self, api_url, timeout = None, retries = 5):
@@ -96,23 +98,31 @@ class Peer(HTTPClient):
 
     def tx(self, txid):
         '''Return a JSON-encoded transaction.'''
-        tx_response = self._get('tx', txid)
-        return response.json()
+        response = self._get('tx', txid)
+        tx = response.json()
+        for tag in tx['tags']:
+            for key in tag:
+                tag[key] = base64url_decode(tag[key].encode())
+        return tx
 
     def tx2(self, txid):
         '''Return a binary-encoded transaction.'''
-        tx_response = self._get('tx2', txid)
-        return tx_response.content
+        response = self._get('tx2', txid)
+        return response.content
 
     def unconfirmed_tx(self, txid):
         '''Return a possibly unconfirmed JSON-encoded transaction.'''
-        tx_response = self._get('unconfirmed_tx', txid)
-        return response.json()
+        response = self._get('unconfirmed_tx', txid)
+        tx = response.json()
+        for tag in tx['tags']:
+            for key in tag:
+                tag[key] = base64url_decode(tag[key].encode())
+        return tx
 
     def unconfirmed_tx2(self, txid):
         '''Return a possibly unconfirmed binary-encoded transaction.'''
-        tx_response = self._get('unconfirmed_tx2', txid)
-        return tx_response.content
+        response = self._get('unconfirmed_tx2', txid)
+        return response.content
 
     def arql(self, logical_expression):
         '''
@@ -152,6 +162,14 @@ class Peer(HTTPClient):
         response = self._post(logical_expression, 'arql')
         return response.json()
 
+    def graphql(self, query):
+        response = self._post({
+            'operationName': None,
+            'query': query,
+            'variables': {}
+        }, 'graphql')
+        return response.json()
+
     def tx_data_html(self, txid):
         '''
         Return the data field of the transaction specified via the transaction ID (hash)
@@ -364,8 +382,8 @@ class Peer(HTTPClient):
 
     def tx_anchor(self):
         '''Return a block anchor to use for building transactions.'''
-        last_tx_response = self._get('tx_anchor')
-        return last_tx_response.text
+        response = self._get('tx_anchor')
+        return response.text
 
     def wallet_txs(self, wallet_address, earliest_tx = None):
         '''
@@ -456,7 +474,14 @@ class Peer(HTTPClient):
         {field} := { 'id' | 'last_tx' | 'owner' | 'tags' | 'target' | 'quantity' | 'data' | 'signature' | 'reward' }
         '''
         response = self._get('tx', hash, field)
-        return response.json()
+        if field == 'tags':
+            tags = response.json()
+            for tag in tags:
+                for key in tag:
+                    tag[key] = base64url_decode(tag[key].encode())
+            return tags
+        else:
+            return response.json()
 
     def tx_id(self, hash):
         '''Return transaction id.'''
@@ -496,7 +521,7 @@ class Peer(HTTPClient):
 
     def height(self):
         '''Return the current block hieght.'''
-        response = self._get('{}/height')
+        response = self._get('height')
         return int(response.text)
 
     def data(self, txid, ext = ''):
@@ -511,3 +536,7 @@ class Peer(HTTPClient):
         '''
         response = self._get(txid + ext)
         return response.content
+
+    def stream(self, txid, ext = ''):
+        response = self._get(txid + ext, stream = True)
+        return response_stream_to_file_object(response)
